#include <LiquidCrystal_I2C.h>
#include <HCSR04.h>
#include <SoftwareSerial.h>

SoftwareSerial bt(12, 13);           // Modul Bluetooth (Tx, Rx)
LiquidCrystal_I2C lcd(0x27, 16, 2);  // Ecran LCD
HCSR04 hc1(6, 7);                    // Senzor HCSR04 (trig pin, echo pin)
HCSR04 hc2(8, 9);                    // Senzor HCSR04 (trig pin, echo pin)

const int motor1pin1 = 2;  // IN1
const int motor1pin2 = 3;  // IN2
const int motor2pin1 = 4;  // IN3
const int motor2pin2 = 5;  // IN4
const int enPin = 10;      // ENA + ENB
const int ledPin = 11;     // Pin LED

int inserted = 0;       // Avem carti in compartimente
int prevInserted = -1;  // Valoarea trecuta lui inserted

int speed_id = 2;                                       // speed id initial
const int speed[6] = { 100, 130, 160, 190, 220, 255 };  // viteze motoare
const int speed_perc[6] = { 50, 60, 70, 80, 90, 100 };  // procentaj viteze
bool startWheels = false;                               // motoarele functioneaza
bool shufflingDisplayed = false;                        // verifica daca mesajul de shuffling e pe ecran

void setup() {
  Serial.begin(9600);
  bt.begin(9600);  // setam baud rate pentru comunicarea software serial
  lcd.init();
  lcd.clear();
  lcd.backlight();

  // Setam toti pinii ca output
  pinMode(enPin, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(motor1pin1, OUTPUT);
  pinMode(motor1pin2, OUTPUT);
  pinMode(motor2pin1, OUTPUT);
  pinMode(motor2pin2, OUTPUT);

  displayWelcomeMessage();
}

void loop() {
  int dist_hc1 = hc1.dist();
  int dist_hc2 = hc2.dist();

  Serial.println(dist_hc1);
  Serial.println(dist_hc2);

  updateInsertedState(dist_hc1, dist_hc2);

  if (inserted != prevInserted && !startWheels) {
    lcd.clear();
    shufflingDisplayed = false;  // Se reseteaza starea shuffling
    if (inserted == 1) {
      digitalWrite(ledPin, HIGH);  // Daca avem carti in slot-uri, aprindem LED-ul
      displaySpeedSelection();     // Afisam mesajul aferent
    } else {
      speed_id = 2;  // Nu avem carti -> Ne ducem in starea initiala
      digitalWrite(ledPin, LOW);
      displayInsertCardMessage();
    }
    prevInserted = inserted;
  }

  if (bt.available() && !startWheels) {
    // avem date de la modulul ble
    char input = bt.read();
    // daca am primit un numar + avem carti -> se schimba viteza
    if (input >= '1' && input <= '6' && !shufflingDisplayed && inserted == 1) {
      speed_id = input - '0';
      updateSpeedDisplay();
    } else if (input == 'S' && inserted == 1) {  // dam start cand am primit S
      displayShufflingMessage();
      startWheels = true;
      startMotors(speed[speed_id - 1]);
    } else if (input == 'S' && inserted == 0) {
      displayErrorMessage();  // Nu avem carti -> afisam eroare
    }
    Serial.println(input);  // afisam caracterul (for debugging)
  }

  // daca distantele sunt mai mari decat 10 si suntem in curs de amestecare, oprim tot
  if (startWheels && dist_hc1 >= 10 && dist_hc2 >= 10) {
    stopShuffling();
  }

  delay(200);
}

/*
  Verificam distantele fata de cei doi senzori
 */
void updateInsertedState(int dist1, int dist2) {
  inserted = (dist1 < 10 && dist2 < 10) ? 1 : 0;
}

/*
  Afisam mesajul de eroare atunci cand primim 'S' de la utilizator
 */
void displayErrorMessage() {
  lcd.clear();
  lcd.setCursor(1, 0);
  lcd.print("Add the cards");
  lcd.setCursor(5, 1);
  lcd.print("first!");
  delay(1000);
  displayInsertCardMessage();
}

/*
  Afisam mesajul de bun venit
 */
void displayWelcomeMessage() {
  lcd.setCursor(1, 0);
  lcd.print("Welcome");
}

/*
  Afisam mesajul cand nu avem nicio carte in compartimente
 */
void displayInsertCardMessage() {
  lcd.setCursor(1, 0);
  lcd.print("Pls add cards");
  lcd.setCursor(1, 1);
  lcd.print("to the slots!");
}

/*
  Afisam mesajul cand avem carti in compartimente si vrem sa modificam
  viteza.
 */
void displaySpeedSelection() {
  lcd.setCursor(2, 0);
  lcd.print("Select speed");
  lcd.setCursor(2, 1);
  lcd.print("Speed: ");
  lcd.print(speed_perc[speed_id - 1]);
  lcd.print("%");
}

/*
 Updateaza procentajul pe ecran (pt functia de mai sus)
 */
void updateSpeedDisplay() {
  lcd.setCursor(9, 1);
  lcd.print(speed_perc[speed_id - 1]);
  lcd.print("%  ");  //  Adaugam spatii pentru a elimina toate caracterele ramase (ex. 100%)
}

/*
  Afisam mesajul cand se amesteca pachetul de carti
 */
void displayShufflingMessage() {
  lcd.clear();
  lcd.setCursor(1, 0);
  lcd.print("Cards are");
  lcd.setCursor(1, 1);
  lcd.print("shuffling...");
  shufflingDisplayed = true;
}

void stopShuffling() {
  startWheels = false;         // oprim motoarele cand distanta e mare
  shufflingDisplayed = false;  // resetam starea de shuffling
  stopMotors();
  doneMessage();
  inserted = 0;
  prevInserted = 1;
}

/*
  Afisam mesajul cand am terminat de amestecat
 */
void doneMessage() {
  lcd.clear();
  lcd.setCursor(1, 0);
  lcd.print("Shuffling done");
  delay(1500);
}

/*
  Porneste motoarele
 */
void startMotors(int motorSpeed) {
  analogWrite(enPin, motorSpeed);  // Se trimite viteza

  digitalWrite(motor1pin1, HIGH);
  digitalWrite(motor1pin2, LOW);

  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
}

/*
  Opreste motoarele
 */
void stopMotors() {
  analogWrite(enPin, 0);  // Disable motors

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, LOW);

  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, LOW);
}
